@@ -502,13 +502,17 @@ class ApplicationSession(BaseSession):
             response = []
             for name, kind in func.__annotations__.items():
                 if name in arguments:
+                    # if a "type" has __origin__ attribute it is a
+                    # parameterized generic
                     if getattr(kind, "__origin__", None) == Union:
+                        expected_types = [arg.__name__ for arg in kind.__args__]
                         if not isinstance(arguments[name], kind.__args__):
                             response.append(
-                                "'{}' required={} got={}".format(name, kind.__name__, type(arguments[name]).__name__))
+                                "'{}' expected types={} got={}".format(name, expected_types,
+                                                                       type(arguments[name]).__name__))
                     elif not isinstance(arguments[name], kind):
                         response.append(
-                            "'{}' required={} got={}".format(name, kind.__name__, type(arguments[name]).__name__))
+                            "'{}' expected type={} got={}".format(name, kind.__name__, type(arguments[name]).__name__))
             if response:
                 raise TypeCheckError(', '.join(response))
             return await txaio.as_future(func, *args, **kwargs)
@@ -560,6 +564,7 @@ class ApplicationSession(BaseSession):
                         authprovider=self._authprovider,
                         authextra=msg.authextra,
                         serializer=self._transport._serializer.SERIALIZER_ID,
+                        transport=self._transport.peer_transport if hasattr(self._transport, 'peer_transport') else None,
                         resumed=msg.resumed,
                         resumable=msg.resumable,
                         resume_token=msg.resume_token,
@@ -1077,15 +1082,21 @@ class ApplicationSession(BaseSession):
                                     self._transport.send(reply)
                                 except SerializationError as e:
                                     # the application-level payload returned from the invoked procedure can't be serialized
-                                    reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.INVALID_PAYLOAD,
-                                                          args=['success return value from invoked procedure "{0}" could not be serialized: {1}'.format(registration.procedure, e)])
-                                    self._transport.send(reply)
+                                    error_reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.INVALID_PAYLOAD,
+                                                                args=['success return value (args={}, kwargs={}) from invoked procedure "{}" could not be serialized: {}'.format(reply.args,
+                                                                                                                                                                                 reply.kwargs,
+                                                                                                                                                                                 registration.procedure,
+                                                                                                                                                                                 e)])
+                                    self._transport.send(error_reply)
                                 except PayloadExceededError as e:
                                     # the application-level payload returned from the invoked procedure, when serialized and framed
                                     # for the transport, exceeds the transport message/frame size limit
-                                    reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.PAYLOAD_SIZE_EXCEEDED,
-                                                          args=['success return value from invoked procedure "{0}" exceeds transport size limit: {1}'.format(registration.procedure, e)])
-                                    self._transport.send(reply)
+                                    error_reply = message.Error(message.Invocation.MESSAGE_TYPE, msg.request, ApplicationError.PAYLOAD_SIZE_EXCEEDED,
+                                                                args=['success return value (args={}, kwargs={}) from invoked procedure "{}" exceeds transport size limit: {}'.format(reply.args,
+                                                                                                                                                                                      reply.kwargs,
+                                                                                                                                                                                      registration.procedure,
+                                                                                                                                                                                      e)])
+                                    self._transport.send(error_reply)
 
                             def error(err):
                                 del self._invocations[msg.request]
@@ -1764,8 +1775,8 @@ class _SessionShim(ApplicationSession):
     ApplicationSession "directly".
 
     **NOTE:** this is not public or intended for use; you should import
-    either :class:`autobahn.asyncio.wamp.Session` or
-    :class:`autobahn.twisted.wamp.Session` depending on which async
+    either autobahn.asyncio.wamp.Session or
+    autobahn.twisted.wamp.Session depending on which async
     framework yo're using.
     """
 
